@@ -26,6 +26,8 @@
 #include <gtk/gtkseparatormenuitem.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnome/gnome-help.h>
+#include <libgnomeui/gnome-ui-init.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gconf/gconf-client.h>
 #include <time.h>
@@ -33,16 +35,25 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <bacon-cd-selection.h>
 
 #include "nautilus-cd-burner.h"
 #include "make-iso.h"
 
+#define MAX_ISO_NAME_LEN 32
+
+enum {
+	CANCEL_NONE,
+	CANCEL_MAKE_ISO,
+	CANCEL_CD_RECORD,
+};
+
+GtkWidget * target_optionmenu_create (void);
+
 static GladeXML *xml;
 
-static GList *recorders = NULL;
-static CancelFunc cancel_func = NULL;
-static gboolean cancel_dangerous = FALSE;
-static gpointer cancel_user_data;
+static CDRecorder *cdrecorder = NULL;
+static int cancel = CANCEL_NONE;
 /* For the image spinning */
 static GList *image_list;
 
@@ -72,16 +83,6 @@ cd_progress_set_text (const char *text)
 	gtk_label_set_text (GTK_LABEL (progress_label), text);
 }
 
-void
-cd_progress_set_cancel_func (CancelFunc _cancel_func,
-			     gboolean _cancel_dangerous,
-			     gpointer user_data)
-{
-	cancel_func = _cancel_func;
-	cancel_dangerous = _cancel_dangerous;
-	cancel_user_data = user_data;
-}
-
 static gboolean
 cd_progress_set_image (gpointer user_data)
 {
@@ -176,14 +177,6 @@ cd_progress_image_cleanup (void)
 	}
 }
 
-static void
-cancel_func_changed_cb (CDRecorder *cdrecorder, CancelFunc _cancel_func,
-			gboolean _cancel_dangerous, gpointer data)
-{
-	cd_progress_set_cancel_func (_cancel_func, _cancel_dangerous,
-				     (gpointer) cdrecorder);
-}
-
 static void
 animation_changed_cb (CDRecorder *cdrecorder, gboolean spinning, gpointer data)
 {
@@ -208,8 +201,6 @@ progress_changed_cb (CDRecorder *cdrecorder, gdouble fraction, gpointer data)
 static void
 reload_dialog_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, TRUE);
@@ -218,8 +209,6 @@ reload_dialog_response (GtkDialog *dialog, gint response_id, gpointer data)
 static void
 insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
@@ -228,8 +217,6 @@ insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
 static void
 insert_cdrw_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
@@ -311,6 +298,7 @@ media_type_matches (CDMediaType type, gboolean blank_cd)
 	switch (type)
 	{
 	case CD_MEDIA_TYPE_ERROR:
+	case CD_MEDIA_TYPE_BUSY:
 		return FALSE;
 	case CD_MEDIA_TYPE_UNKNOWN:
 		return TRUE;
@@ -336,13 +324,13 @@ media_type_matches (CDMediaType type, gboolean blank_cd)
 }
 
 static gboolean
-wait_for_insertion (CDDrive *rec, gboolean blank_cd)
+wait_for_insertion (const CDDrive *rec, gboolean blank_cd)
 {
 	CDMediaType type;
 	gboolean reload;
 
 	reload = FALSE;
-	type = guess_media_type (rec->device);
+	type = cd_drive_get_media_type ((CDDrive *)rec);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		reload = TRUE;
@@ -382,7 +370,7 @@ wait_for_insertion (CDDrive *rec, gboolean blank_cd)
 			return FALSE;
 		}
 
-		type = guess_media_type (rec->device);
+		type = cd_drive_get_media_type ((CDDrive *)rec);
 		reload = FALSE;
 		if (type == CD_MEDIA_TYPE_UNKNOWN
 				|| type == CD_MEDIA_TYPE_ERROR) {
@@ -397,7 +385,6 @@ wait_for_insertion (CDDrive *rec, gboolean blank_cd)
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
 	GtkWidget *dialog, *label, *text, *scrolled_window;
 	GtkTextBuffer *buffer;
 
@@ -482,49 +469,6 @@ estimate_size_callback (const gchar *rel_path,
 	return TRUE;
 }
 
-gboolean
-is_audio (char *mime_type) 
-{
-	if (strcmp (mime_type, "audio/x-wav") == 0 || strcmp (mime_type, "audio/x-ulaw") == 0) {
-		return TRUE;
-	}
-	return FALSE;
-}
-
-static gboolean
-are_all_files_music (char *uri)
-{
-	GnomeVFSFileInfo *info;
-	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult result;
-	gboolean all_music;
-
-	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
-	if (result != GNOME_VFS_OK) {
-		return FALSE;
-	}
-	
-	info = gnome_vfs_file_info_new ();
-	all_music = TRUE;
-
-	while (TRUE) {
-		result = gnome_vfs_directory_read_next (handle, info);
-		if (result != GNOME_VFS_OK) {
-			break;
-		}
-		
-		if (!is_audio (info->mime_type)) {
-			all_music = FALSE;
-			break;
-		}
-	}
-
-	gnome_vfs_directory_close (handle);
-	gnome_vfs_file_info_unref (info);
-
-	return all_music;
-}
-
 static GnomeVFSFileSize
 estimate_size (char *uri)
 {
@@ -541,8 +485,10 @@ estimate_size (char *uri)
 		GList *list;
 
 		if (gnome_vfs_directory_list_load (&list, uri, GNOME_VFS_FILE_INFO_FIELDS_NONE) == GNOME_VFS_OK) {
-			size = 1;
-			gnome_vfs_file_info_list_free (list);
+			if (list != NULL) {
+				size = 1;
+				gnome_vfs_file_info_list_free (list);
+			}
 		}
 	}
 
@@ -562,22 +508,6 @@ setup_close_button (void)
 	g_signal_connect (close_button, "clicked", (GCallback)gtk_main_quit, NULL);
 }
 
-static void
-track_free (Track *track, gpointer user_data)
-{
-	switch (track->type) {
-	case TRACK_TYPE_DATA:
-		g_free (track->contents.data.filename);
-		break;
-	case TRACK_TYPE_AUDIO:
-		g_free (track->contents.audio.filename);
-		g_free (track->contents.audio.cdtext);
-		break;
-	default:
-		g_warning ("Invalid track type %d", track->type);
-	}
-}
-
 static gboolean 
 overwrite_existing_file (GtkWindow *filesel, const char *filename)
 {
@@ -627,91 +557,44 @@ display_error_dialog (GtkWindow *win, const gchar *msg)
 	gtk_widget_destroy (error_dlg);
 }
 
-static void
-file_chooser_response_cb (GtkWidget *chooser, gint response, gpointer data)
-{
-	gchar *file_name;
-	char **file_name_out;
-
-	file_name_out = data;
-
-	if (response != GTK_RESPONSE_OK) {
-		if (response != GTK_RESPONSE_DELETE_EVENT) {
-			gtk_widget_destroy (chooser);
-		}
-		gtk_main_quit ();
-
-		return;
-	}
-
-	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
-
-	if (!file_name)
-		return;
-
-	if (strlen (file_name) < 1) {
-		g_free (file_name);
-		return;
-	}
-
-	/* Change into directory if that's what user selected */
-	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-						     file_name);
-		g_free (file_name);
-	} else {
-		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-			if (!overwrite_existing_file (GTK_WINDOW (chooser), file_name)) {
-				g_free (file_name);
-				return;
-			}
-		}
-
-		gtk_widget_destroy (GTK_WIDGET (chooser));
-		
-		g_free (*file_name_out);
-		*file_name_out = file_name;
-		
-		gtk_main_quit ();
-	}
-}
-
 static char *
 select_iso_filename (void)
 {
 	GtkWidget *chooser;
 	char *filename;
-
-	chooser = gtk_file_chooser_dialog_new (_("Choose a filename for the cdrom image"),
-					       NULL,
-					       GTK_FILE_CHOOSER_ACTION_SAVE,
-					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					       GTK_STOCK_OK, GTK_RESPONSE_OK,
-					       NULL);
+	gint response;
+	chooser = gtk_file_chooser_dialog_new_with_backend (_("Choose a filename for the cdrom image"),
+							    NULL,
+							    GTK_FILE_CHOOSER_ACTION_SAVE,
+							    "unix",
+							    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+							    GTK_STOCK_OK, GTK_RESPONSE_OK,
+							    NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser),
 					   "image.iso");
-
+ reselect:
 	filename = NULL;
-	g_signal_connect (G_OBJECT (chooser),
-			  "response",
-			  G_CALLBACK(file_chooser_response_cb),
-			  &filename);
+	response = gtk_dialog_run (GTK_DIALOG (chooser));
+	
+	if (response == GTK_RESPONSE_OK) {
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+		if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+			if (!overwrite_existing_file (GTK_WINDOW (chooser), filename)) {
+				g_free (filename);
+				goto reselect;
+			}
+		}
 
-	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
-	gtk_window_set_default_size (GTK_WINDOW (chooser), 600, 400);
-
-	gtk_widget_show (chooser);
-
-	/* Run in a loop untill we get the response from file selector */
-	gtk_main ();
+	}
+	gtk_widget_destroy (chooser);
 
 	return filename;
 }
 
 static int
-burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, gboolean music_cd, int speed, gboolean dummy)
+burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy, gboolean debug)
 {
-	CDRecorder *cdrecorder;
+	CDRecorderWriteFlags flags;
 	GList *tracks;
 	char *filename, *str;
 	int fd;
@@ -721,12 +604,6 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 	filename = NULL;
 
 	if (rec->type == CDDRIVE_TYPE_FILE) {
-		if (source_iso != NULL) {
-			/* TODO: display some message here */
-			g_print ("Writing ISO onto ISO, you freak!\n");
-			return RESULT_FINISHED;
-		}
-
 		/* Run the file selector and get the iso file name selected */
 		filename = select_iso_filename ();
 
@@ -758,26 +635,28 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			}
 		}
 	} else {
-		if (source_iso == NULL && !music_cd) {
+		if (source_iso == NULL) {
 			GConfClient *gc;
 			const char *path;
 
 			gc = gconf_client_get_default ();
 			path = gconf_client_get_string (gc, "/apps/nautilus-cd-burner/temp_iso_dir", NULL);
 			g_object_unref (G_OBJECT (gc));
-			if (path == NULL || strcmp (path, "") == 0) {
+			if (path != NULL && path[0] != '\0') {
 				filename = g_build_filename (path, "image.iso.XXXXXX", NULL);
-				fd = mkstemp(filename);
+				fd = g_mkstemp(filename);
 				close (fd);
 			}
-		} else if (!music_cd) {
+		} else {
 			filename = g_strdup (source_iso);
 		}
 	}
 
-	if (source_iso == NULL && !music_cd) {
+	if (source_iso == NULL) {
+		cancel = CANCEL_MAKE_ISO;
+
 		if (filename != NULL) {
-			res = make_iso (filename, label, FALSE, TRUE);
+			res = make_iso (filename, label, FALSE, TRUE, debug);
 		} else {
 			res = RESULT_RETRY;
 		}
@@ -785,21 +664,23 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		if (res == RESULT_RETRY) {
 			g_free (filename);
 			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
-			fd = mkstemp(filename);
+			fd = g_mkstemp(filename);
 			close (fd);
 
-			res = make_iso (filename, label, FALSE, TRUE);
+			res = make_iso (filename, label, FALSE, TRUE, debug);
 			if (res == RESULT_RETRY) {
 				g_free (filename);
 				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
-				fd = mkstemp(filename);
+				fd = g_mkstemp(filename);
 				close (fd);
 
-				res = make_iso (filename, label, TRUE, TRUE);
+				res = make_iso (filename, label,
+						TRUE, TRUE, debug);
 			}
 
 			if (!res) {
 				/* User cancelled or we had an error. */
+				cancel = CANCEL_NONE;
 				goto out;
 			}
 		}
@@ -812,11 +693,14 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		setup_close_button ();
 		gtk_main ();
 	} else {
+		Track *track;
+
 		if (!wait_for_insertion (rec, blank_cd)) {
 			return RESULT_CANCEL;
 		}
 
 		cdrecorder = cd_recorder_new ();
+		cancel = CANCEL_CD_RECORD;
 
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "progress-changed",
@@ -830,10 +714,6 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "animation-changed",
 				  G_CALLBACK (animation_changed_cb),
 				  NULL);
-		g_signal_connect (G_OBJECT (cdrecorder),
-				  "cancel-func-changed",
-				  G_CALLBACK (cancel_func_changed_cb),
-				  NULL);
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "insert-cd-request",
 				  G_CALLBACK (insert_cd_request_cb),
@@ -842,17 +722,28 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-rwcd-request",
 				  G_CALLBACK (insert_rwcd_request_cb),
 				  NULL);
-		
-		if (music_cd) {
-			tracks = build_list_of_audio_tracks ();
-		} else {
-			Track *track;
-			track = g_new0 (Track, 1);
-			track->type = TRACK_TYPE_DATA;
-			track->contents.data.filename = g_strdup (filename);
-			tracks = g_list_prepend (tracks, track);
+
+		track = g_new0 (Track, 1);
+		track->type = TRACK_TYPE_DATA;
+		track->contents.data.filename = g_strdup (filename);
+		tracks = g_list_prepend (tracks, track);
+
+		flags = 0;
+		if (eject_cd) {
+			flags |= CDRECORDER_EJECT;
 		}
-		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy, FALSE);
+		if (blank_cd) {
+			flags |= CDRECORDER_BLANK;
+		}
+		if (dummy) {
+			flags |= CDRECORDER_DUMMY_WRITE;
+		}
+		if (debug) {
+			flags |= CDRECORDER_DEBUG;
+		}
+
+		res = cd_recorder_write_tracks (cdrecorder, (CDDrive *)rec,
+				tracks, speed, flags);
 
 		if (source_iso == NULL) {
 			unlink (filename);
@@ -862,32 +753,31 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			cd_progress_set_text (_("Complete"));
 			setup_close_button ();
 			gtk_main ();
-		} else {
+		} else if (res != RESULT_CANCEL) {
 			show_error_message (cdrecorder);
 		}
-		
+
+		cancel = CANCEL_NONE;
 		g_object_unref (cdrecorder);
 	}
 	
  out:
 	g_free (filename);
-	g_list_foreach (tracks, (GFunc)track_free, NULL);
+	g_list_foreach (tracks, (GFunc)cd_recorder_track_free, NULL);
 	g_list_free (tracks);
 	return res;
 }
 
 
-static CDDrive *
+static const CDDrive *
 lookup_current_recorder (GladeXML *xml)
 {
-	GtkWidget *option_menu;
-	GtkWidget *item;
-	CDDrive *rec;
+	GtkWidget *bcs;
+	const CDDrive *rec;
+
+	bcs = glade_xml_get_widget (xml, "target_optionmenu");
+	rec = bacon_cd_selection_get_cdrom (BACON_CD_SELECTION (bcs));
 
-	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
-	item = gtk_menu_get_active (GTK_MENU (GTK_OPTION_MENU (option_menu)->menu));
-	rec = g_object_get_data (G_OBJECT (item), "cd-drive");
-	
 	return rec;
 }
 
@@ -900,13 +790,11 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *item;
 	GtkWidget *eject_check;
 	GtkWidget *blank_check;
-	GtkWidget *music_check;
-	GtkWidget *name_entry;
 	GList *l, *children;
 	GConfClient *gc;
 	char *name;
 	int i, default_speed;
-	CDDrive *rec;
+	const CDDrive *rec;
 
 	/* Find active recorder: */
 	rec = lookup_current_recorder (xml);
@@ -951,49 +839,56 @@ refresh_dialog (GladeXML *xml)
 	/* Disable blank for burn-to-iso */
 	blank_check = glade_xml_get_widget (xml, "blank_check");
 	gtk_widget_set_sensitive (blank_check, rec->type & CDDRIVE_TYPE_CDRW_RECORDER);
-
-	/* Disable music for burn-to-iso */
-	music_check = glade_xml_get_widget (xml, "music_check");
-	gtk_widget_set_sensitive (music_check, rec->type != CDDRIVE_TYPE_FILE);
-
-	name_entry = glade_xml_get_widget (xml, "cdname_entry");
-	gtk_widget_set_sensitive (name_entry,
-				  rec->type == CDDRIVE_TYPE_FILE ||
-				  !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check)));
-
 }
 
 static void
-target_changed (GtkOptionMenu *option_menu, gpointer data)
+target_changed (BaconCdSelection *bcs, GladeXML *xml)
 {
-  GladeXML *xml = data;
-
   refresh_dialog (xml);
 }
 
 static void
-music_cd_toggled (GtkToggleButton *button, gpointer data)
+cdname_entry_insert_text (GtkEditable *editable,
+			  const gchar *new_text,
+			  gint         new_text_length,
+			  gint        *position)
 {
-  GladeXML *xml = data;
+	char *current_text;
 
-  refresh_dialog (xml);
+	if (new_text_length < 0) {
+		new_text_length = strlen (new_text);
+	}
+
+	current_text = gtk_editable_get_chars (editable, 0, -1);
+	if (strlen (current_text) +  new_text_length >= MAX_ISO_NAME_LEN) {
+		gdk_display_beep (gtk_widget_get_display (GTK_WIDGET (editable)));
+		g_signal_stop_emission_by_name (editable, "insert_text");
+	}
+		
+	g_free (current_text);
 }
 
+GtkWidget *
+target_optionmenu_create (void)
+{
+	GtkWidget *widget;
+
+	widget = bacon_cd_selection_new ();
+	g_object_set (widget, "file-image", TRUE, NULL);
+	gtk_widget_show (widget);
+
+	return widget;
+}
 
 static void
-init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean disable_reuse)
+init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean from_iso)
 {
-	GList *l;
-	CDDrive *rec;
 	GtkWidget *option_menu;
-	GtkWidget *menu;
-	GtkWidget *item;
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *reuse;
-	GtkWidget *music_check;
 	int size_mb;
-	char *str;
+	char *str, *last;
 	char buf[129];
 	GDate *now;
 
@@ -1004,32 +899,22 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 				size_mb);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	
+
 	/* Fill in targets */
 	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
-	menu = gtk_option_menu_get_menu	(GTK_OPTION_MENU (option_menu));
-	for (l = recorders; l != NULL; l = l->next) {
-		rec = l->data;
-		if (rec->type == CDDRIVE_TYPE_FILE) {
-			item = gtk_separator_menu_item_new ();
-			gtk_widget_show (item);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		}
-		item = gtk_menu_item_new_with_label (rec->display_name);
-		g_object_set_data (G_OBJECT (item), "cd-drive", rec);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	g_signal_connect (option_menu, "device-changed",
+			G_CALLBACK (target_changed), xml);
 
-	g_signal_connect (option_menu, "changed", (GCallback)target_changed, xml);
-	
 	/* Set default name */
-	
 	entry = glade_xml_get_widget (xml, "cdname_entry");
-	if (disc_name != NULL) {
-		gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
+	g_signal_connect (entry, "insert_text", G_CALLBACK (cdname_entry_insert_text),
+			  NULL);
+	if (from_iso != FALSE) {
+		gtk_widget_set_sensitive (entry, FALSE);
+		if (disc_name != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
+			gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
+		}
 	} else {
 		now = g_date_new ();
 
@@ -1037,54 +922,73 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 
 		/*
 		translators: see strftime man page for meaning of %b, %d and %Y
-		the maximum length for this field is 32 characters
+		the maximum length for this field is 32 bytes
 		*/
 		g_date_strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), now);
 
 		g_date_free (now);
+
+		/* Cut off at 32 bytes */
+		last = str = buf;
+		while (*str != 0 &&
+		       (str - buf) < MAX_ISO_NAME_LEN) {
+			last = str;
+			str = g_utf8_next_char (str);
+		}
+		if (*str != 0) {
+			*last = 0;
+		}
+		
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
 
 	reuse = glade_xml_get_widget (xml, "reuse_check");
 	g_assert (GTK_IS_WIDGET (reuse));
-	gtk_widget_set_sensitive (reuse, !disable_reuse);
+	gtk_widget_set_sensitive (reuse, !from_iso);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reuse), TRUE);
 
-	music_check = glade_xml_get_widget (xml, "music_check");
-	if (!are_all_files_music ("burn:///")) {
-		gtk_widget_hide (music_check);	
-	}
-	g_signal_connect (music_check, "toggled", (GCallback)music_cd_toggled, xml);
-
-	
 	refresh_dialog (xml);
 }
 
+static gboolean
+ask_cancel (void)
+{
+	GtkWidget *warning_dialog;
+	int res;
+
+	warning_dialog = gtk_message_dialog_new (cd_progress_get_window (),
+						 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_WARNING,
+						 GTK_BUTTONS_YES_NO,
+						 _("Are you sure you want to cancel the CD write operation?\n"
+						 "Some CD writers may require that you restart the machine to get it working again."));
+	gtk_window_set_title (GTK_WINDOW (warning_dialog),
+			      _("Are you sure you want to cancel?"));
+	res = gtk_dialog_run (GTK_DIALOG (warning_dialog));
+	gtk_widget_destroy (warning_dialog);
+
+	return (res == GTK_RESPONSE_YES);
+}
+
 static void
 do_cancel (void)
 {
-	GtkWidget *warning_dialog;
-	gint res;
-	
-	if (cancel_dangerous) {
-		warning_dialog = gtk_message_dialog_new (cd_progress_get_window (),
-							 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-							 GTK_MESSAGE_WARNING,
-							 GTK_BUTTONS_YES_NO,
-							 _("Are you sure you want to cancel the CD write operation?\n"
-							   "Some CD writers may require that you restart the machine to get it working again."));
-		gtk_window_set_title (GTK_WINDOW (warning_dialog),
-				      _("Are you sure you want to cancel?"));
-		res = gtk_dialog_run (GTK_DIALOG (warning_dialog));
-		gtk_widget_destroy (warning_dialog);
-		if (res != GTK_RESPONSE_YES) {
+	if (cancel == CANCEL_NONE) {
+		return;
+	}
+
+	if (cancel == CANCEL_MAKE_ISO) {
+		make_iso_cancel ();
+		return;
+	}
+
+	if (cd_recorder_cancel (cdrecorder, TRUE) != FALSE) {
+		if (ask_cancel () == FALSE) {
 			return;
 		}
 	}
 
-	if (cancel_func != NULL) {
-		(*cancel_func) (cancel_user_data);
-	}
+	cd_recorder_cancel (cdrecorder, FALSE);
 }
 
 static gboolean
@@ -1101,6 +1005,21 @@ cancel_clicked (GtkButton *button, gpointer data)
 	do_cancel ();
 }
 
+static void
+help_activate (GtkWindow *parent)
+{
+	GError *err = NULL;
+
+	if (gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml", "gosnautilus-475", &err) == FALSE) {
+		char *msg;
+
+		msg = g_strdup_printf (_("There was a problem displaying the help contents: %s."), err->message);
+		display_error_dialog (parent, msg);
+		g_error_free (err);
+		g_free (msg);
+	}
+}
+
 static gboolean
 verify_iso (const char *filename, char **iso_label)
 {
@@ -1181,15 +1100,15 @@ my_gdk_pixbuf_new_from_stock (const char *name)
 int
 main (int argc, char *argv[])
 {
-	CDDrive *rec;
-	GtkWidget *dialog, *entry, *eject_check, *blank_check, *music_check, *reuse_check, *progress_window;
+	const CDDrive *rec;
+	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
-	int res, speed;
-	gboolean eject, blank, music, reuse, dummy;
+	int res, speed, i;
+	gboolean eject, blank, reuse, dummy, debug;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
-	char *source_iso, *disc_name = NULL;
+	char *source_iso, *disc_name;
 	struct stat stat_buf;
 	GConfClient *gc;
 	GdkPixbuf *icon;
@@ -1197,10 +1116,14 @@ main (int argc, char *argv[])
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
+
+	disc_name = NULL;
+
+	gnome_program_init ("nautilus-cd-burner", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
+			    GNOME_PARAM_APP_DATADIR, SHAREDIR,
+			    GNOME_PARAM_NONE);
     
-	recorders = scan_for_cdroms (TRUE, TRUE);
-	
-	gtk_init (&argc, &argv);
 	gnome_vfs_init ();
 
 	if (argc > 2) {
@@ -1216,7 +1139,13 @@ main (int argc, char *argv[])
 		return 1;
 	}
 
-	source_iso = (argc == 2) ? argv[1] : NULL;
+	source_iso = NULL;
+	for (i = 1; i < argc; i++) {
+		if (strncmp ("--", argv[i], 2) != 0) {
+			source_iso = argv[i];
+			break;
+		}
+	}
 
 	if (source_iso != NULL) {
 		if (!verify_iso (source_iso, &disc_name)) {
@@ -1259,29 +1188,59 @@ main (int argc, char *argv[])
 			return 1;
 		}
 	}
-	
+#if 0
+	recorders = scan_for_cdroms (TRUE, source_iso != NULL ? FALSE : TRUE);
+
+	if (recorders == NULL) {
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("You do not seem to have any CD recorder in your machine. Please verify your setup."));
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+		gtk_window_set_title (GTK_WINDOW (dialog),
+				      _("No CD recorder available"));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		return 1;
+	}
+#endif
 	xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	dialog = glade_xml_get_widget (xml, "cdr_dialog");
 
 	init_dialog (xml, dialog, size, disc_name, source_iso != NULL);
 
-	if (disc_name != NULL) g_free (disc_name);
+	if (disc_name != NULL) {
+		g_free (disc_name);
+	}
 
 	gtk_widget_show (dialog);
 
 	dummy_check = glade_xml_get_widget (xml, "dummy_check");
 
-#ifndef ENABLE_DUMMY_WRITE
-	gtk_widget_hide (dummy_check);
-#endif
+	gc = gconf_client_get_default ();
+	debug = gconf_client_get_bool (gc, "/apps/nautilus-cd-burner/debug", NULL);
+	g_object_unref (G_OBJECT (gc));
+
+	if (debug) {
+		gtk_widget_hide (dummy_check);
+	}
 
 	icon = my_gdk_pixbuf_new_from_stock ("gtk-cdrom");
 	gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+	progress_window = glade_xml_get_widget (xml, "progress_window");
+	gtk_window_set_icon (GTK_WINDOW (progress_window), icon);
 	gdk_pixbuf_unref (icon);
-	res = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	do {
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		if (res == GTK_RESPONSE_HELP) {
+			help_activate (GTK_WINDOW (dialog));
+		}
+	} while (res == GTK_RESPONSE_HELP);
 
 	gtk_widget_hide (dialog);
-	progress_window = glade_xml_get_widget (xml, "progress_window");
 	cd_progress_image_setup ();
 	gtk_widget_show (progress_window);
 	cancel_button = glade_xml_get_widget (xml, "cancel_button");
@@ -1290,7 +1249,7 @@ main (int argc, char *argv[])
 
 	if (res == 0) {
 		rec = lookup_current_recorder (xml);
-		
+
 		entry = glade_xml_get_widget (xml, "cdname_entry");
 		label = gtk_entry_get_text (GTK_ENTRY (entry));
 
@@ -1300,15 +1259,12 @@ main (int argc, char *argv[])
 		blank_check = glade_xml_get_widget (xml, "blank_check");
 		blank = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blank_check));
 
-		music_check = glade_xml_get_widget (xml, "music_check");
-		music = (rec->type != CDDRIVE_TYPE_FILE) &&
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check));
-	
 		dummy = FALSE;
 
-#ifdef ENABLE_DUMMY_WRITE
-		dummy = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dummy_check));
-#endif
+		if (debug) {
+			dummy = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON (dummy_check));
+		}
 
 		option_menu = glade_xml_get_widget (xml, "speed_optionmenu");
 		item = gtk_menu_get_active (GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu))));
@@ -1317,7 +1273,7 @@ main (int argc, char *argv[])
 		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
 		g_object_unref (G_OBJECT (gc));
 
-		res = burn_cd (rec, source_iso, label, eject, blank, music, speed, dummy);
+		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy, debug);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
